// compile: mpic++ -o mpi2.ex mpi2.cpp
// run: time mpirun -np 2 ./mpi2.ex

#include <mpi.h>

#include <iostream>
#include <iomanip>
#include <cstdlib>
using namespace std;

double compute_pi(int N) {
  int M=0;
  for(int n=0;n<N;n++) {
    const double x=((double)rand())/((double)RAND_MAX);
    const double y=((double)rand())/((double)RAND_MAX);
    if (x*x+y*y<=1.0) M++;
    }
  return 4.0*((double)M)/((double)N);
  }

int main(int argc, char *argv[])
  {
  int rank, nranks;
 
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &nranks);

  int N=1000000;
  double local_pi, global_pi=0.0; 
  srand(rank);
  local_pi=compute_pi(N);
  
  MPI_Reduce(&local_pi, &global_pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

  if (rank==0) cout << setprecision(10) << fixed << "pi(" << nranks << ")=" << global_pi/((double)nranks) << endl;
  
  MPI_Finalize();
  
  return 0;
  }